@@ -1,6 +1,7 @@
 """
 ModelAdmin code to display polymorphic models.
 """
+import sys
 from django import forms
 from django.conf.urls import patterns, url
 from django.contrib import admin
@@ -27,6 +28,9 @@ except ImportError:
     def add_preserved_filters(context, form_url):
         return form_url
 
+if sys.version_info[0] >= 3:
+    long = int
+
 
 __all__ = (
     'PolymorphicModelChoiceForm', 'PolymorphicParentModelAdmin',
@@ -116,7 +120,7 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
     #: The regular expression to filter the primary key in the URL.
     #: This accepts only numbers as defensive measure against catch-all URLs.
     #: If your primary key consists of string values, update this regular expression.
-    pk_regex = '(\d+)'
+    pk_regex = '(\d+|__fk__)'
 
 
     def __init__(self, model, admin_site, *args, **kwargs):
@@ -271,14 +275,23 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
         real_admin = self._get_real_admin(object_id)
         return real_admin.delete_view(request, object_id, extra_context)
 
+    def get_preserved_filters(self, request):
+        if '_changelist_filters' in request.GET:
+            request.GET = request.GET.copy()
+            filters = request.GET.get('_changelist_filters')
+            f = filters.split("&")
+            for x in f:
+                c = x.split('=')
+                request.GET[c[0]] = c[1]
+            del request.GET['_changelist_filters']
+        return super(PolymorphicParentModelAdmin, self).get_preserved_filters(request)
 
     def get_urls(self):
         """
         Expose the custom URLs for the subclasses and the URL resolver.
         """
         urls = super(PolymorphicParentModelAdmin, self).get_urls()
-        meta = self.model._meta
-        info = meta.app_label, getattr(meta, 'model_name', meta.module_name)
+        info = _get_opt(self.model)
 
         # Patch the change URL so it's not a big catch-all; allowing all custom URLs to be added to the end.
         # The url needs to be recreated, patching url.regex is not an option Django 1.4's LocaleRegexProvider changed it.
@@ -314,7 +327,10 @@ class PolymorphicParentModelAdmin(admin.ModelAdmin):
             # See if the path started with an ID.
             try:
                 pos = path.find('/')
-                object_id = long(path[0:pos])
+                if pos == -1:
+                    object_id = long(path)
+                else:
+                    object_id = long(path[0:pos])
             except ValueError:
                 raise Http404("No ct_id parameter, unable to find admin subclass for path '{0}'.".format(path))
 
@@ -545,3 +561,10 @@ class PolymorphicChildModelAdmin(admin.ModelAdmin):
                 except ValueError:
                     pass   # field not found in form, Django will raise exception later.
         return subclass_fields
+
+
+def _get_opt(model):
+    try:
+        return model._meta.app_label, model._meta.model_name  # Django 1.7 format
+    except AttributeError:
+        return model._meta.app_label, model._meta.module_name
